require 'csv'
require_relative 'project'
require_relative 'funding_round'

module CrowdFund
  class FundRequest
    attr_reader :name
  
    def initialize(name)
      @name = name.capitalize
      @projects = []
    end
  
    def load_projects(filename)
      CSV.foreach(filename) do |line|
        add_project(Project.from_csv(line))
      end
    end
  
    def save_unfunded_projects(to_file = "unfunded_projects.txt")
      File.open(to_file, "w") do |file|
        file.puts "#{@name} Unfunded Projects:"
        under_funded = @projects.filter { |p| p.funded? == false }
        under_funded.sort.each { |uf| file.puts uf.name_and_funding }
      end
    end
  
    def add_project(project)
      @projects << project
    end

    def request_funding(rounds)
      puts "There are #{@projects.size} projects in the #{@name} project list:"
      puts @projects
    
      1.upto(rounds) do |round|
        puts "\nRound: #{round}"
        @projects.each do |project|
          FundingRound.one_round(project)
        end
      end
  
      print_stats
    end

    def print_stats
      puts "\nFunding Results"
  
      fully_funded, under_funded = @projects.partition { |p| p.funded? }
      puts "\n#{fully_funded.size} projects funded"
      fully_funded.each {|ff| puts ff.name_and_funding}
  
      puts "\n#{under_funded.size} projects under funded"
      under_funded.sort.each {|uf| puts uf.name_and_funding}
  
      @projects.sort.each do |project|
        puts "\nProject #{project.name}'s pledges:"
        project.each_donated_pledge do |pledge|
          puts "£#{pledge.amount} in #{pledge.name} pledges"
        end
        puts "£#{project.pledges} in total pledges"
      end
    end
  end
end
